(*
	Graphics.m
		Graphics routines for FeynArts
		last modified 23 Oct 00 th
*)

Begin["`Graphics`"]

NPi = N[Pi]

	(* for "PS" format: paper size in bp for finding margins *)
PaperSize = {595, 842}	(* A4, use 72 {8.5, 11} for Letter *)

DefaultImageSize = 72 {6, 7}

	(* Dimensions of a single diagram *)
DiagramBorder = 1

DiagramCanvas = 20

DiagramSize = DiagramCanvas + 2 DiagramBorder

LabelFontSize = 2 (* units of the diagram coordinate system *)

LabelFont = "Helvetica"

	(* for ordinary vertices and counter terms: *)
PropagatorThickness = Thickness[.11 / DiagramSize]

VertexThickness = PointSize[8 .11 / DiagramSize]

CounterThickness = Thickness[3 .11 / DiagramSize]

	(* for counter terms: *)
CrossWidth = 1

ArrowLength = 1.2

	(* height of an arrow's thick end over the baseline *)
ArrowHeight = .4

DampingConst = (.65 ArrowLength)^4

ScalarDashing = Dashing[{.66, .66} / DiagramSize]

GhostDashing = Dashing[{.11, .66} / DiagramSize]

	(* # of points per unit arc length used in drawing Sine and
	   Cycles lines *)
NPoints = 10

	(* # of Sine or Cycles wave crests per unit arc length *)
NCrestsSine = .5

NCrestsCycles = .65

SineAmp = .25

CyclesAmp = .6

	(* the breadth of the "spikes" of the cycloid *)
CyclesBreadth = .16

	(* default radius for label positioning, must be the
	   same as DEFAULT_RADIUS in TopologyEditor.java *)
DefaultRadius = 1.3


Options[ Paint ] = {
  PaintLevel -> InsertionLevel,
  ColumnsXRows -> 3,
  AutoEdit -> True,
  SheetHeader -> Automatic,
  Numbering -> Full,
  FieldNumbers -> False,
  DisplayFunction :> $DisplayFunction
}

Paint::nolevel =
"Warning: Level `1` is not contained in this insertion."

Paint::colrows =
"ColumnsXRows is not an integer or a pair of integers."

Paint[ top:P$Topology, opt___Rule ] :=
  Paint[ TopologyList[top], opt ]

Paint[ top:(P$Topology -> _), opt___Rule ] :=
  Paint[ TopologyList[][top], opt ]

Paint[ tops_TopologyList, options___Rule ] :=
Block[ {fnum, ghead, opt = ActualOptions[Paint, options]},
  fnum = FieldNumbers /. opt;
  ghead = Switch[ ghead = SheetHeader /. opt,
    None | False,
      FeynArtsGraphics[],
    Automatic | True, 
      ghead = #[[0, 1]]&/@ tops[[1]];
      FeynArtsGraphics[
        Count[ghead, Incoming | External] -> Count[ghead, Outgoing] ],
    _,
      FeynArtsGraphics[ghead]
  ];

  PaintSheet[tops]
]

Paint[ tops:TopologyList[info___][__], options___Rule ] :=
Block[ {plevel, ins, ghead,
fnum = False, opt = ActualOptions[Paint, options]},
  If[ (plevel = ResolveLevel[PaintLevel /. opt /. {info} /.
        Options[InsertFields]]) === $Failed,
    Return[$Failed] ];
  ins = PickLevel[plevel][tops];
  If[ FreeQ[plevel, Generic],
    ins = Select[
      ins /. Insertions[Generic][ gr__ ] :> Join@@ TakeIns/@ {gr},
      Length[#] =!= 0 & ] ];
  Scan[ If[FreeQ[ins, #], Message[Paint::nolevel, #]]&, plevel ];

  If[ InitializeModel[ Model /. {info} /. Options[InsertFields],
    GenericModel -> (GenericModel /. {info} /. Options[InsertFields]),
    Reinitialize -> False ] === $Failed, Return[$Failed] ];

  ghead = Switch[ ghead = SheetHeader /. opt,
    None | False,
      FeynArtsGraphics[],
    Automatic | True, 
      If[ (ghead = Process /. {info}) === Process, FeynArtsGraphics[],
        FeynArtsGraphics[Map[TheLabel, ghead, {2}]]
      ],
    _,
      FeynArtsGraphics[ghead]
  ];

  PaintSheet[
    ins //. (x:Graph[___][__] -> Insertions[_][gr___]) :> Seq[x, gr] ]
]


PaintSheet[ tops_ ] :=
Block[ {auto, disp, cols, rows, dhead, g, topnr = 0, runnr = 0},
  auto = AutoEdit /. opt /. {False -> 2, _ -> 1};
  Switch[ cols = ColumnsXRows /. opt,
    _Integer,
      rows = cols,
    {_Integer, _Integer},
      {cols, rows} = cols,
    _,
      Message[Paint::colrows]; cols = rows = 3
  ];
  Switch[ Numbering /. opt,
    Full,
      dhead[gr_] := 
      Block[ {specs},
        ++runnr;
        specs = Cases[Head[gr], lev_ == n_ :>
          {" ", StringTake[ToString[lev], 1], ToString[n]}];
        If[ Length[specs] =!= 0,
          specs = {specs, " N", ToString[runnr]} ];
        DiagramGraphics["T" <> ToString[topnr] <> specs]
      ],
    Simple,
      dhead[_] := DiagramGraphics[ ToString[++runnr] ],
    _,
      dhead[_] = DiagramGraphics[]
  ];

  g = Flatten[TopologyGraphics/@ List@@ tops] /. _Index -> Null;
  g = Flatten[{g, Table[Null, {Mod[rows cols - Length[g], rows cols]}]}];
  g = ghead@@ Partition[Partition[g, cols], rows];
  (DisplayFunction /. opt)[g];
  g
]


TopologyGraphics[ top_ -> gr_ ] :=
Block[ {ginfo, gtop, vertexplot},
  ginfo = Shape[gtop = top /. Vertex[e_, _] -> Vertex[e], auto];
  gtop = Transpose[{
    List@@ gtop /. ginfo[[1]],
    ginfo[[2]],
    ginfo[[3]] }];
  vertexplot = VGraphics/@ Vertices[top] /. ginfo[[1]];
  ++topnr;
  FAPrint[2, "> Top. ", topnr, ": ", Pluralize[{Length[gr]}, " diagram"]];
  dhead[#]@@ Flatten[{
    Apply[PGraphics, gtop /. List@@ # /. Field[_] -> 0, 1],
    vertexplot }]&/@ List@@ gr
]

TopologyGraphics[ top_ ] :=
  TopologyGraphics[
    MapIndexed[Append[ Take[#1, 2], #2[[1]] ]&, top] -> {{}} ] /; fnum

TopologyGraphics[ top_ ] :=
  TopologyGraphics[ Append[Take[#, 2], 0]&/@ top -> {{}} ]


VGraphics[ _[e_, c_:0][n_] ] := VertexGraphics[c][ Vertex[e][n] ]

PGraphics[ _[from_, to_, 0, ___], height_, labelpos_ ] :=
  PropagatorGraphics[Straight][from, to, height, labelpos]

PGraphics[ _[from_, to_, particle_, ___], height_, labelpos_ ] :=
  PropagatorGraphics[
    PropagatorType[particle],
    TheLabel[particle],
    PropagatorArrow[particle] ][from, to, height, labelpos]


Format[ DiagramGraphics[h___][__] ] := SequenceForm["[", h, "]"]

Format[ FeynArtsGraphics[h___][l__List] ] :=
  FeynArtsGraphics[h]@@ MatrixForm/@ {l}


Unprotect[Show, Display, Export]

Show[ g:FeynArtsGraphics[___][___] ] := Show/@ Render[g]

Display[ file_, g:FeynArtsGraphics[___][___], format___String,
  opt___?OptionQ ] :=
Block[ {rg},
  rg = Render[g, InferFormat[file, format],
    ImageSize -> (ImageSize /. {opt} /. Options[Display])];
  MapThread[Display[##, format, opt]&,
    {FilePerSheet[file, Length[rg]], rg}]
]

Display[ file_, s_String, ___ ] := (WriteString[file, s]; Close[file])

Export[ file_, g:FeynArtsGraphics[___][___], format___String,
  opt___?OptionQ ] :=
Block[ {rg},
  rg = Render[g, InferFormat[file, format],
    ImageSize -> (ImageSize /. {opt} /. Options[Export])];
  MapThread[Export[##, format, opt]&,
    {FilePerSheet[file, Length[rg]], rg}]
]

Export[ file_, s_String, ___ ] := (WriteString[file, s]; Close[file])

Protect[Show, Display, Export]


InferFormat[ _, format_ ] = format

InferFormat[ file_String ] := "PS" /; StringMatchQ[file, "*.ps"]

InferFormat[ file_String ] := "EPS" /; StringMatchQ[file, "*.eps"]

InferFormat[ file_String ] := "TeX" /; StringMatchQ[file, "*.tex"]

InferFormat[ _ ] = Sequence[]


Options[Render] = {ImageSize -> Automatic}

getsize[ opt___, def_ ] :=
  If[NumberQ[#], {#, #}, #]& @
    Round[ImageSize /. ActualOptions[Render, opt] /. Automatic -> def]

prologue := prologue =
  ReadList[ToFileName[{$FeynArtsDir, "FeynArts"}, "FeynArts.pro"],
    Record, RecordSeparators -> ""][[1]]

epsf = ""

Render[ g:FeynArtsGraphics[___][___], "EPS", opt___Rule ] :=
Block[ {PaperSize = imgsize, epsf = " EPSF-3.0"},
  Flatten[ Render[Head[g][#], "PS", opt]&/@ List@@ g ]
]

Render[ g:FeynArtsGraphics[___][___], "PS", opt___Rule ] :=
Block[ {imgsize = getsize[opt, DefaultImageSize], bbox,
None = 0, Forward = 1, Backward = -1},
  bbox = Round[.5 (PaperSize - imgsize)];
  bbox = {bbox, bbox + imgsize};
  { PSString[ "\
%!PS-Adobe-3.0", epsf, "\n\
%%BoundingBox: 0 0 ", PaperSize, "\n\
%%Pages: ", Length[g], "\n",
    prologue,
    MapIndexed[
      { "\n%%Page: ", #2, #2, "\ngsave\n",
        bbox, #1, "\ngrestore\nshowpage\n" }&,
      PSRender[Head[g]/@ List@@ g] ], "\n\
%%Trailer\n\
end\n\
%%EOF\n" ] }
]

Render[ g:FeynArtsGraphics[___][___], "TeX", opt___Rule ] :=
Block[ {imgsize = getsize[opt, DefaultImageSize],
None = 0, Forward = 1, Backward = -1},
  { "\\unitlength=1bp%\n\n" <> TeXRender[Head[g]/@ List@@ g] }
]

Render[ g:FeynArtsGraphics[___][___], opt___ ] :=
Block[ {imgsize = getsize[opt, {288, 288}],
None = 0, Forward = 1, Backward = -1},
  MmaRender[Head[g]/@ List@@ g]
]


FilePerSheet[ file_, 1 ] := {file}

FilePerSheet[ file:{__}, n_ ] := Thread[FilePerSheet[#, n]&/@ file]

FilePerSheet[ file_String, n_ ] :=
Block[ {p, pre, post},
  p = StringPosition[file, "."];
  If[ Length[p] === 0, pre = file <> "_"; post = "",
    p = p[[-1, 1]] - 1;
    pre = StringTake[file, p] <> "_";
    post = StringDrop[file, p] ];
  Array[pre <> ToString[#] <> post &, n]
] /; file =!= "stdout"

FilePerSheet[ file_, n_ ] := Table[file, {n}]


PSString[ s_String ] = s

PSString[ s_Symbol ] := "/" <> ToString[s] <> " "

PSString[ n_ ] := ToString[n] <> " "

PSString[ s__ ] := StringJoin[PSString/@ Flatten[{s}]]


TeXString[ {x_, y_} ] := "(" <> ToString[x] <> "," <> ToString[y] <> ")"

TeXString[ s_ ] := ToString[s]

TeXString[ s__ ] := StringJoin[TeXString/@ {s}]


Attributes[PSRender] = Attributes[TeXRender] =
  Attributes[MmaRender] = {Listable}


PSRender[ FeynArtsGraphics[h___][sheet_] ] :=
Block[ {rows, cols, g},
  {rows, cols} = Dimensions[sheet];
  g = PSRender[{Title[h], sheet}];
  PSString[cols, rows, "Layout\n" <> g]
]

Attributes[ TeXJoin ] = {Flat, OneIdentity}

TeXJoin[ s1_String, s2_String ] := s1 <> "\\quad " <> s2

TeXRender[ FeynArtsGraphics[in_ -> out_][sheet_] ] :=
  TeXRender[
    FeynArtsGraphics[TeXJoin@@ Flatten[{in, "\\to", out}]][sheet] ]

TeXRender[ FeynArtsGraphics[h___][sheet_] ] :=
Block[ {rows, cols, g},
  {rows, cols} = Dimensions[sheet];
  g = TeXRender[{Title[h], sheet}];
  TeXString["\\begin{feynartspicture}", imgsize, {cols, rows}, "\n" <>
    g <> "\\end{feynartspicture}\n\n"]
]

MmaRender[ FeynArtsGraphics[h___][sheet_] ] :=
Block[ {rows, cols, fsize, g, title,
(* magnify the labels a bit for screen viewing: *)
LabelFontSize = 1.26 LabelFontSize},
  {rows, cols} = Dimensions[sheet];
  g = MapIndexed[DiagramBox, sheet, {2}];
  title = MmaRender[Title[h]];
  fsize = LabelFontSize Min[imgsize / {cols, rows}] / DiagramSize;
  Graphics[ Flatten[{g, title}],
    PlotRange -> {{0, cols}, {0, rows}} DiagramSize,
    AspectRatio -> rows/cols]
]

DiagramBox[ Null, _ ] = {}

DiagramBox[ g_, {yoff_, xoff_} ] :=
  Rectangle[ {xoff - 1, rows - yoff} DiagramSize,
             {xoff, rows - yoff + 1} DiagramSize, MmaRender[g] ]

Title[ ] = {}

Title[ t_ ] := (
  rows += .3;
  LabelText[t, {.5 cols, rows - .12} DiagramSize, {0, 0}, 1.2, 0] )


PSRender[ DiagramGraphics[h___][pv__] ] :=
  "\n(" <> h <> ") Diagram\n" <> Transpose[PSRender[{pv}]]

TeXRender[ DiagramGraphics[h___][pv__] ] :=
  "\n\\FADiagram{" <> h <> "}\n" <> Transpose[TeXRender[{pv}]]

MmaRender[ DiagramGraphics[h___][pv__] ] :=
Block[ {g = Transpose[MmaRender[{pv}]]},
  Graphics[
    Flatten[{
      PropagatorThickness, g[[1]],
      scope[ VertexThickness, g[[2]] ],
      MmaRender[LabelText[h, {.5 DiagramCanvas, -.5}, {0, -1}, .8]]
    }] /. scope[a__] :> scope@@ Flatten[{a}] /. scope -> List,
    PlotRange -> {{0, DiagramSize}, {0, DiagramSize}} - DiagramBorder,
    AspectRatio -> 1 ]
]

PSRender[ Null ] = "\n() Diagram\n"

TeXRender[ Null ] = "\n\\FADiagram{}\n"


PSRender[ VertexGraphics[cto_][xy_] ] :=
  {{}, PSString[xy, cto, "Vert\n"]}

TeXRender[ VertexGraphics[cto_][xy_] ] :=
  {{}, TeXString["\\FAVert", xy, "{", cto, "}\n"]}

MmaRender[ VertexGraphics[0][xy_] ] := {{}, Point[xy]}

MmaRender[ VertexGraphics[c_?Negative][xy_] ] := {{},
  { scope[ GrayLevel[Max[0, 1.1 + .3 c]], Disk[xy, CrossWidth] ],
    Circle[xy, CrossWidth] }}

MmaRender[ VertexGraphics[c_][xy_] ] := {{},
  { scope[ CounterThickness,
      Line[{xy - .5 CrossWidth, xy + .5 CrossWidth}],
      Line[{xy - {.5, -.5} CrossWidth, xy + {.5, -.5} CrossWidth}] ],
    Array[Circle[xy, (.25 # + .8) CrossWidth]&, c - 1] }}


PSRender[ PropagatorGraphics[type_, label_:0, arrow_:0][
  from_, to_, height_, labelpos_:0 ] ] :=
Block[ {dir, ommc, cs, ctr, rad, mid, dphi, line},
  line = PSString["[ ", type, "] ", arrow, height, from, to, "Prop\n"];

  If[ label =!= 0,
    CalcPropData[from, to, height];
    line = line <> PSRender[PropLabel[label, labelpos, arrow, type]] ];

  {line, {}}
]

TeXRender[ PropagatorGraphics[type_, label_:0, arrow_:0][
  from_, to_, height_, labelpos_:0 ] ] :=
Block[ {dir, ommc, cs, ctr, rad, mid, dphi, line},
  line = TeXString["\\FAProp", from, to,
    If[ NumberQ[height], {height, ""}, height], "{" <>
    StringDrop[StringJoin[(" /" <> ToString[#])&/@ Flatten[{type}]], 1] <>
    "}{", arrow, "}\n"];

  If[ label =!= 0,
    CalcPropData[from, to, height];
    line = line <> TeXRender[PropLabel[label, labelpos, arrow, type]] ];

  {line, {}}
]

MmaRender[ PropagatorGraphics[type_, label_:0, arrow_:0][
  from_, to_, height_, labelpos_:0 ] ] :=
Block[ {dir, ommc, cs, ctr, rad, mid, dphi, line, phi, damping, h, v},
  CalcPropData[from, to, height];
  If[ arrow =!= 0,
    damping[phi_] :=
      Block[ {x = (rad Abs[phi - ommc])^4}, x / (x + DampingConst) ],
  (* else *)
    damping[phi_] = 1 ];
  phi = Mod[ommc - dphi, 2 NPi];
  h = Flatten[{type}];
  dphi *= 2. / Length[h];
  line = HalfLine[#, phi += dphi, dphi]&/@ h;

  If[ arrow =!= 0,
    h = .5 arrow ArrowLength {Cos[dir], Sin[dir]};
    v = ArrowHeight cs;
    line = {line, Polygon[{mid + h, mid - h + v, mid - h - v}]} ];

  If[ label =!= 0,
    line = {line, MmaRender[PropLabel[label, labelpos, arrow, type]]} ];

  {line, {}}
]


(* CalcPropData computes the data necessary for actual drawing:
   dir  -- the direction along the propagator
   ommc -- the direction of the perpendicular bisector
   ctr  -- the center of the circle on which the prop lies,
   rad  -- the radius of the circle
   mid  -- the position on the middle of the prop
           (the blue square in the topology editor)
   dphi -- half the opening angle *)

CalcPropData[ from_, _, xy_List ] := (
  mid = xy;
  ctr = .5 (from + mid);
  rad = Distance[from, ctr];
  ommc = Orientation[from, ctr];
  dir = ommc + .5 NPi;
  cs = {Cos[ommc], Sin[ommc]};
  dphi = NPi
)

CalcPropData[ from_, to_, height_ ] :=
Block[ {lab, h},
  lab = .5 Distance[from, to];
  dir = Orientation[from, to];
  ommc = dir + If[height < 0, .5, -.5] NPi;
  cs = {Cos[ommc], Sin[ommc]};
  ctr = mid = .5 (from + to);
  If[ height != 0,
    dphi = 2. ArcTan[h = Abs[height]];
    rad = lab / Sin[dphi];
    dphi *= Sign[height];
    mid += h lab cs;
    h = If[h > 1., -1, 1],
  (* else *)
    rad = 20000.;
    dphi = ArcSin[lab / rad];
    h = 1
  ];
  ctr -= h Sqrt[rad^2 - lab^2] cs
]


HalfLine[ Sine, phi_, dphi_ ] :=
Block[ {arc, w, n},
  arc = rad Abs[dphi];
  w = 2. NPi (.5 + Max[1, Round[NCrestsSine arc]]);
  Line[
    Table[ arc = phi - n dphi;
      ctr + (rad - damping[arc] SineAmp Sin[n w]) {Cos[arc], Sin[arc]},
      {n, 0, 1, 1. / Floor[NPoints arc]} ] ]
]

rshift = CyclesAmp - SineAmp

phadj = ArcCos[rshift / CyclesAmp]

sphadj = Sin[phadj]

HalfLine[ Cycles, phi_, dphi_ ] :=
Block[ {arc, w, n, phamp},
  arc = rad Abs[dphi];
  w = 2. (phadj + NPi Max[1, Round[NCrestsCycles arc]]);
  phamp = CyclesBreadth NPi Sign[dphi] / rad;
  Line[
    Table[ arc = n w - phadj;
      ctr + (rad + CyclesAmp Cos[arc] - rshift) *
        Through[{Cos, Sin}[
          phi - n dphi - phamp (Sin[arc] - (2 n - 1) sphadj)]],
      {n, 0, 1, 1. / Floor[2 NPoints arc]} ] ]
]

HalfLine[ Straight, phi_, dphi_ ] :=
  If[ rad < 20000,
    Circle[ctr, rad, Sort[{phi - dphi, phi}]],
    Line[{ ctr + rad {Cos[phi - dphi], Sin[phi - dphi]},
           ctr + rad {Cos[phi], Sin[phi]} }] ]

HalfLine[ ScalarDash, phi_, dphi_ ] :=
  scope[ ScalarDashing, HalfLine[Straight, phi, dphi] ]

HalfLine[ GhostDash, phi_, dphi_ ] :=
  scope[ GhostDashing, HalfLine[Straight, phi, dphi] ]


PropLabel[ label_, labelpos_, arrow_, type_ ] :=
Block[ {rad, phi, s},
  {rad, phi} =
    If[ NumberQ[labelpos], {labelpos DefaultRadius, 0.}, labelpos ];
  s = Sign[rad Cos[phi]];
  cs *= s;
  Which[
    arrow =!= 0 || !FreeQ[type, Sine],
      mid += SineAmp cs,
    !FreeQ[type, Cycles],
      mid += If[s < 0, CyclesAmp + rshift, SineAmp] cs ];
  phi += ommc;
  cs = -Round[1.3 cs];
  LabelText[ label,
    mid + rad {Cos[phi], Sin[phi]} + .24 LabelFontSize cs,
    cs, 1, dir ]
]


LabelText[ in_ -> out_, pos_, align_, size_, dir_ ] :=
Block[ {t, l, cs = {Cos[dir], Sin[dir]}},
  t = Flatten[{in, "\\to", out}];
  l = .5 (Length[t] + 1);
  MapIndexed[LabelText[#1, pos + 4.5 (#2[[1]] - l) cs, align, size]&, t]
]

LabelText[ t_List, pos_, align_, size_, dir_ ] :=
Block[ {l = .5 (Length[t] + 1), cs = {Cos[dir], Sin[dir]}},
  MapIndexed[LabelText[#1, pos + 2 (#2[[1]] - l) cs, align, size]&, t]
]

PSRender[ LabelText[t_, pos_, align_, size_, ___] ] :=
  PSString[pos, align, size, "(" <> ToPS[t] <> ") Label\n"]


TeXRender[ LabelText[t_, pos_, align_, size_, ___] ] :=
Block[ {ComposedChar = TeXComposedChar},
  TeXString[
    "\\FALabel", pos, "[" <> Extract[texalign, align + {2, 2}] <> "]{" <>
    Which[size > 1, "\\large ", size < 1, "\\small ", True, ""] <>
    "$", t, "$}\n" ]
]

texalign = {{"bl", "l", "tl"},
            {"b",  "",  "t"},
            {"br", "r", "tr"}}

TeXComposedChar[ t_, sub_:Null, super_:Null, over_:Null ] :=
Block[ {tex = t},
  If[ sub =!= Null, tex = tex <> "_" <> ToString[sub] ];
  If[ super =!= Null, tex = tex <> "^" <> ToString[super] ];
  If[ over =!= Null, tex = ToString[over] <> " " <> tex ];
  tex
]

If[ $Notebooks,

MmaRender[ LabelText[t_, r__] ] :=
  NotebookChar[Flatten[{ToUnicode[t]}], r];

NotebookChar[ {t_, sub_:" ", super_:" ", over_:" "},
  pos_, align_, size_, ___ ] :=
Block[ {label = t},
  If[ over =!= " ", label = OverscriptBox[label, over] ];
  Which[
    sub =!= " " && super =!= " ",
      label = SubsuperscriptBox[label, sub, super],
    sub =!= " ",
      label = SubscriptBox[label, sub],
    super =!= " ",
      label = SuperscriptBox[label, super]
  ];
  Text[
    StyleForm[DisplayForm[label], FontFamily -> LabelFont,
      FontSize -> size fsize],
    pos, align ]
],

(* else $Notebooks *)

MmaRender[ LabelText[t_, r__] ] :=
  MapIndexed[ KernelChar[##, r]&, Flatten[{ToPS[t]}] ];

KernelChar[ " ", __ ] = {};

KernelChar[ t_, {n_}, pos_, align_, size_, ___ ] :=
Block[ {newpos, newalign},
  {newpos, newalign} = If[ size < 1, {pos, align},
    {pos + .24 size LabelFontSize (palign[[n]] - align), talign[[n]]} ];
  Text[
    If[ StringTake[t, 1] === "$",
      FontForm[StringDrop[t, 1], {"Symbol", sizes[[n]] size fsize}],
      FontForm[t, {LabelFont, sizes[[n]] size fsize}] ],
    newpos, newalign ]
];

sizes = {1, .667, .667, 1};

palign = {{0, 0}, {1.2, -.9}, {1.2, .7}, {0, 1.2}};

talign = {{0, 0}, {-1, 0}, {-1, 0}, {0, -1}};

] (* endif $Notebooks *)


pr[ Loop[l_] ][ from_, to_, ___ ] := (
  AppendTo[loops, l];
  AppendTo[props[l], {from, to}] )

pr[ type_ ][ from_, to_, ___ ] := (
  AppendTo[props[Tree], {from, to}];
  AppendTo[props[type], {from, to}] )

AutoShape[ top_ ] :=
Block[ {ginfo, props, ext, l, tree, mesh, mesh2, tadbr, vars, vert,
tad, min, ok, c, ct, pt, shrink = {}, rev = {}, loops = {}},

  props[ _ ] = {};
  top /. Propagator -> pr;
  loops = Union[loops];

  Off[FindMinimum::fmmp, FindMinimum::fmcv, FindMinimum::precw,
    FindMinimum::fmgz];

	(* a) fix the incoming and outgoing propagators on the left and
	      right side, respectively *)
  ginfo = Join[
    l = Length[props[Incoming]] / 20.;
    MapIndexed[
      #1[[1]] -> {0, 20 - Round[(#2[[1]] - .5) / l]}&,
      props[Incoming] ],
    l = Length[props[Outgoing]] / 20.;
    MapIndexed[
      #1[[1]] -> {20, 20 - Round[(#2[[1]] - .5) / l]}&,
      props[Outgoing] ] ];

	(* b) shrink loops to 1 point which is the center of an imaginary
	      circle on which the external points of the loop lie *)
  vert = Flatten[props[Tree]];
  tree = Fold[
    ( l = Flatten[props[#2]];
      ext[#2] = l = Select[vert, MemberQ[l, #]&];
      AppendTo[ rev, c = center[Length[l]][#2] -> Union[l] ];
      shrink = Join[ shrink, c = Thread[Reverse[c]] ];
      #1 /. c )&,
    props[Tree], loops ];

	(* c) cut tadpole-like parts and minimize the length of the
	      remaining mesh of propagators *)
  mesh2 = Leaves@@ Apply[twig, tree /. ginfo, 1] /. twig -> List;
  tadbr[ _ ] = {};
  Cases[mesh2, branch[ctr:center[_][_], v_, ___] :>
    (tadbr[ctr] = Flatten[{tadbr[ctr], v /. rev}]), Infinity];
  mesh = mesh2 /. branch[__] :> Seq[];
  vert = Cases[mesh, leaf[a_] -> a];
  mesh = mesh /. leaf[___] :> Seq[];
  mesh = List@@ Fold[
    #1 /. {
      Leaves[{a___, #2, b___}, {#2, c_}] :> Leaves[{a, b, #2, c}],
      Leaves[{a___, #2, b___}, {c_, #2}] :> Leaves[{a, b, #2, c}] } &,
    mesh,
    vert = Union[Join[ vert, Cases[mesh, _[2, ___][_], {2}] ]] ] /.
    twig -> List;
  vert = (# -> CartesianVar[#])&/@
    Complement[Cases[mesh, _[__][_], {2}], vert];
  If[ Length[vert] =!= 0,
    dist = Apply[Distance2, mesh /. vert, {1}];
    dist = Expand[Apply[Plus, Outer[(#1 - #2)^2 &, dist, dist], {0, 1}]];
    vars = Flatten[Last/@ vert];
    min := FindMinimum@@ Prepend[{#, 10 + Random[]}&/@ vars, dist];
    Do[
      If[ ok = (Head[c = min] =!= FindMinimum &&
                Min[ l = Last/@ c[[2]] ] >= 0 &&
                Max[l] <= 20), Break[] ],
    {5} ];
    vert = vert /.
      If[ ok, MapAt[Round, #, 2]&/@ c[[2]], (# -> RandInt)&/@ vars ]
  ];
  ginfo = Join[ginfo, vert];
  ginfo = Flatten[ {ginfo,
    (l = (#[[-1]] - (c = #[[1]])) / (Length[#] - 1);
     MapIndexed[#1 -> c + l #2[[1]] &, Take[#, {2, -2}]])&/@
      (Select[mesh, Length[#] > 2 &] /. ginfo)} ];

	(* d) minimizing the straight distance of a tadpole to its
	      nearest vertex v would make the tadpole stick to v.
	      Therefore, the tadpole is given polar coordinates with
	      fixed radius and we try to construct the ideal angle with
	      respect to the lines joining at v by maximizing. *)
  mesh2 = (List@@ mesh2) /.
    leaf[br__] :> Sequence@@ Cases[{br}, branch[__]] /. ginfo;
  While[ Length[ tad = Cases[mesh2, branch[{_, _}, __]] ] =!= 0,
    mesh2 = Fold[FixTad, mesh2, tad] ];

	(* e) for each loop, distribute the external points of the loop
	      at the middle of the line from the center to the external
	      vertex and distribute the remaining points of the loop
	      on the imaginary circle around the center. *)
  ok = ginfo;
  Scan[
    Function[rul,
      If[ Length[ vert = Union[ext[ rul[[1, 1]] ]] ] === 1,
        c = SetTadpole[vert[[1]], rul],
        SetMiddle[#, rul]&/@ vert; c = rul[[2]] ];
      SetLoop[props[ rul[[1, 1]] ] /. ginfo, c] ],
    Select[ginfo, !FreeQ[#, center]&] ];

	(* f) last resort: randomize any remaining vertex *)
  ginfo = Join[ MapAt[Clip, #, 2]&/@ ginfo,
    (# -> {RandInt, RandInt})&/@
      Union[Cases[top /. ginfo, Vertex[__][_], Infinity]] ];

	(* g) give tadpoles and identical propagators curvature so that
	      they do not fall on top of each other *)
  pt[ _[Loop[n_]][v_, v_], _ ] :=
    center[ Length[ext[n]] ][n] /. ginfo /. center[_][_] :>
      Clip[ (v /. ginfo) + 4 Through[{Cos, Sin}[2. NPi Random[]]] ];
  pt[ _, 0 ] = 0;
  ct[ p_ ] := If[ (c = (Count[top, p] - 1) / 2) === 0, 0,
    pt[ p, n_ ] = .8 n / c;
    ct[ p ] = c
  ];

  On[FindMinimum::fmmp, FindMinimum::fmcv, FindMinimum::precw,
    FindMinimum::fmgz];

  { Select[ginfo, FreeQ[#, center]&],
    pt[#, ct[#]--]&/@ List@@ top,
    Table[1, {Length[top]}] }
]


Attributes[twig] = {Orderless}

twig[ a:_[1, ___][_], b_ ] := branch[b, a]

Attributes[Leaves] = {Orderless, Flat}

Leaves[ branch[a_, b__], twig[a:_[2, ___][_], c_] ] :=
  Leaves[branch[c, a, b]]

Leaves[ br:branch[a_, __].., tw:twig[a_, _].. ] :=
  Switch[ Length[{tw}],
    1, Leaves[branch[ Sequence@@ DeleteCases[tw, a], a, br ]],
    2, Leaves[leaf[br, a], tw],
    _, Leaves[leaf[br], tw]
  ]

cutbranch[ vert__, br___branch ] :=
  Sequence[ br, Drop[{vert}, {2, -2}] ]


Clip[ xy_ ] := Max[Min[#, 20], 0]&/@ N[xy]

RandInt := Plus@@ Table[Random[Integer, 9], {2}] + 1


Distance[ p1_, p2_ ] := Block[ {d = p2 - p1}, Sqrt[d . d] ]

Distance2[ p1_, ___, p2_ ] := Block[ {d = p2 - p1}, d . d ]

Orientation[ p1_, p2_ ] := N[ArcTan@@ (p2 - p1)]


CartesianVar[ n_ ] := CartesianVar[n] = {Unique["X"], Unique["Y"]}


FixTad[ mesh_, br_ ] :=
Block[ {stem, root, c, phi, dphi, vert, min, dist},
  stem = DeleteCases[List@@ br, branch[__]];
  c = (root = stem[[1]]) + (Length[stem] + 1) {Cos[phi], Sin[phi]};
  vert = Cases[mesh, {a___, root, b___} :> Seq[a, b]];
  Switch[ Length[vert],
    0, phi = 0,
    1, phi = Orientation[vert[[1]], root],
    _, dist = -Plus@@ (Distance[c, #]&)/@ vert;
       Do[
         If[ Head[min = FindMinimum[dist, {phi, 1.57 + dphi}]] =!=
               FindMinimum &&
             Min[ min = c /. min[[2]] ] >= 0 &&
             Max[min] <= 20, c = min; Break[] ],
       {dphi, 0, 2. NPi, .5 NPi} ];
       phi = 2. NPi Random[]
  ];
  If[ Min[c] < 0 || Max[c] > 20, c = {RandInt, RandInt} ];
  c = (c - root)/(Length[stem] - 1);
  vert = MapIndexed[#1 -> root + #2[[1]] c &, Rest[stem]];
  ginfo = Join[ginfo, vert];
  mesh /. br -> cutbranch@@ br /. vert
]


SetLoop[ loop_, ctr_ ] :=
Block[ {vert, vars, rad, angle, off, min, c = 0},
  vert = Union[ Cases[loop, _[__][_], {2}] ];
  If[ Length[vert] =!= 0,
    rad = Union[ Cases[loop, _List, {2}] ];
    rad = If[ Length[rad] === 0, 5,
      Plus@@ (Distance[#, ctr]&/@ rad) / Length[rad] ];
    angle = 2. NPi / Length[vert];
    vars = (# ->
      ctr + rad Through[{Cos, Sin}[++c angle + off]])&/@ vert;
    min = FindMinimum@@ {
      -Plus@@ Apply[Distance, loop /. vars, {1}],
      {off, 2. NPi Random[]} };
    ginfo = Join[ ginfo,
      If[ Head[min] === FindMinimum, vars /. off -> 2. NPi Random[],
        c = vars /. min[[2]];
        If[ Length[Intersection[
          Round[Last/@ c], Round[Last/@ ginfo] ]] === 0,
          c,
          # -> ctr + Random[] rad Through[{Sin, Cos}[2. NPi Random[]]]&/@
            vert ] ]
    ]
  ];
]


SetMiddle[ vert_, ctr_ -> xy_ ] :=
Block[ {ex, mid},
  ex = DeleteCases[
    Flatten[Select[props[Tree], !FreeQ[#, vert]&]], vert ];
  If[ Length[ex] =!= 0,
    If[ Length[ mid = Complement[ex, tadbr[ctr]] ] =!= 0, ex = mid ];
    ex = ex /. shrink /. ginfo;
    While[ Length[ex] =!= 0 && (mid = Plus@@ ex / Length[ex]) == xy,
      ex = Rest[ex] ];
    If[ Length[ex] === 0, mid = {RandInt, RandInt} ];
    AppendTo[ginfo, vert -> .6 xy + .4 mid]
  ]
]


SetTadpole[ vert_, ctr_ -> xy_ ] :=
Block[ {adj, max, new, a1, a2},
  adj = Select[tree, !FreeQ[#, ctr]&] /. ok;
  ginfo = ginfo /. ctr -> vert;
  If[ Length[adj] === 1,
    new = 2.6 xy - .8 Plus@@ adj[[1]],
  (* else *)
    adj = Apply[Orientation, adj /. {a_, xy} -> {xy, a}, {1}];
    max = -1;
    Outer[
      If[ (new = Abs[#2 - #1]) > max, max = new; a1 = #1; a2 = #2 ]&,
      adj, adj ];
    new = xy + 4 Through[{Cos, Sin}[.5 (a1 + a2)]] ];
  If[ Distance2[xy, max = Clip[new]] < 2.8,
    ginfo = ginfo /. xy -> xy - .7 (new - max) ];
  AppendTo[ginfo, ctr -> max];
  max
]


ToJava[ p__, n_?NumberQ ] := ToJava[p, {n DefaultRadius, 0}]

ToJava[ _[from_, from_], {xc_, yc_}, {xl_, yl_} ] :=
  {-from, xc, yc, xl, yl}

ToJava[ _[from_, to_], height_, {xl_, yl_} ] :=
  {from, to, height, xl, yl}


(* call the topology editor *)

Shape::wait =
"Starting Java and the topology editor. This may take a moment."

Shape::notopedit =
"Could not load the topology editor. Make sure you have J/Link and Java
installed."

Shape::javaerror =
"Could not open a topology-editor window."

Shape[ tops:TopologyList[___][___] | TopologyList[___] ] :=
Block[ {remaining = Length[tops]},
  MapIndexed[
    (--remaining; FAPrint[ 2, "> Top. ", #2[[1]] ]; Shape[#1])&,
    List@@ tops ]
]

Shape[ top:P$Topology -> _ ] := Shape[top]

Shape[ top:P$Topology, auto_:0 ] :=
Block[ {edittop, gname, ghead, ginfo, arg1, arg2, exitcode},
  edittop = Take[#, 2]&/@ Topology@@ top /.
    External -> Incoming /. Vertex[e_, _] -> Vertex[e];
  ghead = ShortHand[ #[[0, 1]] ]&/@ edittop;
  gname = StringJoin[
    # <> ToString[Count[ghead, #]] &/@ {"Inc", "Out", "Int", "Loop"} ];
  ghead = ToExpression[gname];
  res = auto;
  If[ Head[ginfo = ghead[edittop]] =!= List,
	(* attempt to load GraphInfo *)
    If[ !MemberQ[$LoadedGraphInfos, gname],
      Off[Get::noopen];
      Get[ ToFileName[$TopologyDataDir, gname <> ".m"] ];
      On[Get::noopen];
      AppendTo[$LoadedGraphInfos, gname];
    ];
    If[ Head[ginfo] =!= List,
      --res; Set@@ {ginfo, AutoShape[edittop]} ];
  ];
  If[ res > 0, Return[ginfo] ];

  If[ editorclass === False,
    Message[Shape::wait];
    Needs["JLink`"];
    JLink`InstallJava[ JLink`ClassPath ->
      ToFileName[{$FeynArtsDir, "FeynArts"}, "TopologyEditor.jar"] ];
    editorclass = JLink`LoadClass["de.FeynArts.TopologyEditor"];
    If[ Head[editorclass] =!= JLink`JavaClass, Message[Shape::notopedit] ]
  ];
  If[ Head[editorclass] =!= JLink`JavaClass, Return[ginfo] ];

  If[ !JLink`JavaObjectQ[editor],
    editor = JLink`JavaNew[editorclass];
    If[ !JLink`JavaObjectQ[editor],
      Message[Shape::javaerror];
      Return[ginfo] ];
  ];

  arg1 = Last/@ ginfo[[1]];
  arg2 = MapThread[ ToJava,
    { List@@ edittop /. MapIndexed[ First/@ Rule[##]&, ginfo[[1]] ],
      ginfo[[2]],
      ginfo[[3]] } ];
  editor@putGraphInfo[ N[Flatten[arg1]], N[Flatten[arg2]] ];

  exitcode = JLink`DoModal[];
  If[ exitcode === 0,
    ginfo = MapAt[ MapThread[Rule, {First/@ ginfo[[1]], #}]&,
      editor@getGraphInfo[], 1 ];
    (#1[#2] = #3)&[ghead, edittop, ginfo];
    $LoadedGraphInfos = Union[Append[$LoadedGraphInfos, gname]];
    $ModifiedGraphInfos =
      Union[Append[$ModifiedGraphInfos, gname]];
    System`$Epilog := SaveGraphInfos[] ];

  If[ remaining === 0 || exitcode === 2,
    editor@closeWindow[];
    JLink`ReleaseObject[editor];
    editor = False;
    If[exitcode === 2, Abort[]];
  ];

  ginfo
]


editorclass = editor = False

remaining = 0


$LoadedGraphInfos = $ModifiedGraphInfos = {}

SaveGraphInfos[ ] := (
  Put[ Definition[#], ToFileName[$TopologyDataDir, # <> ".m"] ]&/@
    $ModifiedGraphInfos;
  $ModifiedGraphInfos = {} )


ShortHand[ Loop[_] ] = "Loop"

ShortHand[ type_ ] := StringTake[ ToString[type], 3 ]


ToPS[ "\\alpha" ] = "$a";
ToPS[ "\\beta" ] = "$b";
ToPS[ "\\gamma" ] = "$g";
ToPS[ "\\delta" ] = "$d";
ToPS[ "\\epsilon" ] = "$e";
ToPS[ "\\varepsilon" ] = "$e";
ToPS[ "\\zeta" ] = "$z";
ToPS[ "\\eta" ] = "$h";
ToPS[ "\\theta" ] = "$q";
ToPS[ "\\vartheta" ] = "$J";
ToPS[ "\\iota" ] = "$i";
ToPS[ "\\kappa" ] = "$k";
ToPS[ "\\lambda" ] = "$l";
ToPS[ "\\mu" ] = "$m";
ToPS[ "\\nu" ] = "$n";
ToPS[ "\\xi" ] = "$x";
ToPS[ "\\pi" ] = "$p";
ToPS[ "\\varpi" ] = "$v";
ToPS[ "\\rho" ] = "$r";
ToPS[ "\\varrho" ] = "$r";
ToPS[ "\\sigma" ] = "$s";
ToPS[ "\\varsigma" ] = "$V";
ToPS[ "\\tau" ] = "$t";
ToPS[ "\\upsilon" ] = "$u";
ToPS[ "\\phi" ] = "$f";
ToPS[ "\\varphi" ] = "$j";
ToPS[ "\\chi" ] = "$c";
ToPS[ "\\psi" ] = "$y";
ToPS[ "\\omega" ] = "$w";
ToPS[ "\\Gamma" ] = "$G";
ToPS[ "\\Delta" ] = "$D";
ToPS[ "\\Theta" ] = "$Q";
ToPS[ "\\Lambda" ] = "$L";
ToPS[ "\\Xi" ] = "$X";
ToPS[ "\\Pi" ] = "$P";
ToPS[ "\\Sigma" ] = "$S";
ToPS[ "\\Upsilon" ] = "$\241";
ToPS[ "\\Phi" ] = "$F";
ToPS[ "\\Psi" ] = "$X";
ToPS[ "\\Omega" ] = "$W";
ToPS[ "\\infty" ] = "$\245";
ToPS[ "\\pm" ] = "$\261";
ToPS[ "\\partial" ] = "$\266";
ToPS[ "\\leq" ] = "$\243";
ToPS[ "\\geq" ] = "$\263";
ToPS[ "\\times" ] = "$\264";
ToPS[ "\\otimes" ] = "$\304";
ToPS[ "\\oplus" ] = "$\305";
ToPS[ "\\nabla" ] = "$\321";
ToPS[ "\\neq" ] = "$\271";
ToPS[ "\\equiv" ] = "$\272";
ToPS[ "\\approx" ] = "$\273";
ToPS[ "\\ldots" ] = "$\274";
ToPS[ "\\in" ] = "$\316";
ToPS[ "\\notin" ] = "$\317";
ToPS[ "\\sim" ] = "$\176";
ToPS[ "\\sqrt" ] = "$\326";
ToPS[ "\\propto" ] = "$\265";
ToPS[ "\\subset" ] = "$\314";
ToPS[ "\\supset" ] = "$\311";
ToPS[ "\\subseteq" ] = "$\315";
ToPS[ "\\supseteq" ] = "$\312";
ToPS[ "\\bullet" ] = "$\267";
ToPS[ "\\perp" ] = "$^";
ToPS[ "\\simeq" ] = "$@";
ToPS[ "\\vee" ] = "$\332";
ToPS[ "\\wedge" ] = "$\331";
ToPS[ "\\leftrightarrow" ] = "$\253";
ToPS[ "\\leftarrow" ] = "$\254";
ToPS[ "\\rightarrow" ] = ToPS[ "\\to" ] = "$\256";
ToPS[ "\\uparrow" ] = "$\255";
ToPS[ "\\downarrow" ] = "$\257";
ToPS[ "\\Leftarrow" ] = "$\334";
ToPS[ "\\Rightarrow" ] = "$\336";
ToPS[ "\\Uparrow" ] = "$\335";
ToPS[ "\\Downarrow" ] = "$\337";
ToPS[ "\\bar" ] = "\305";
ToPS[ "\\hat" ] = "\303";
ToPS[ "\\tilde" ] = "\304";
ToPS[ "\\dot" ] = "\307";
ToPS[ "\\ddot" ] = "\310";
ToPS[ "\\vec" ] = "$\256";
ToPS[ "\\prime" ] = "'";
ToPS[ "\\#" ] = "#";
ToPS[ "\\&" ] = "&";
ToPS[ "\\$" ] = "$";
ToPS[ "\\%" ] = "%";
ToPS[ "\\_" ] = "_";
ToPS[ "-" ] = "$-";
ToPS[ Null ] = " ";
ToPS[ ComposedChar[t__] ] := ToPS/@ {t};
ToPS[ c_ ] := ToString[c]

ToUnicode[ "\\alpha" ] = "\[Alpha]";
ToUnicode[ "\\beta" ] = "\[Beta]";
ToUnicode[ "\\gamma" ] = "\[Gamma]";
ToUnicode[ "\\delta" ] = "\[Delta]";
ToUnicode[ "\\epsilon" ] = "\[Epsilon]";
ToUnicode[ "\\varepsilon" ] = "\[CurlyEpsilon]";
ToUnicode[ "\\zeta" ] = "\[Zeta]";
ToUnicode[ "\\eta" ] = "\[Eta]";
ToUnicode[ "\\theta" ] = "\[Theta]";
ToUnicode[ "\\vartheta" ] = "\[CurlyTheta]";
ToUnicode[ "\\iota" ] = "\[Iota]";
ToUnicode[ "\\kappa" ] = "\[Kappa]";
ToUnicode[ "\\lambda" ] = "\[Lambda]";
ToUnicode[ "\\mu" ] = "\[Mu]";
ToUnicode[ "\\nu" ] = "\[Nu]";
ToUnicode[ "\\xi" ] = "\[Xi]";
ToUnicode[ "\\pi" ] = "\[Pi]";
ToUnicode[ "\\varpi" ] = "\[CurlyPi]";
ToUnicode[ "\\rho" ] = "\[Rho]";
ToUnicode[ "\\varrho" ] = "\[CurlyRho]";
ToUnicode[ "\\sigma" ] = "\[Sigma]";
ToUnicode[ "\\varsigma" ] = "\[FinalSigma]";
ToUnicode[ "\\tau" ] = "\[Tau]";
ToUnicode[ "\\upsilon" ] = "\[Upsilon]";
ToUnicode[ "\\phi" ] = "\[Phi]";
ToUnicode[ "\\varphi" ] = "\[CurlyPhi]";
ToUnicode[ "\\chi" ] = "\[Chi]";
ToUnicode[ "\\psi" ] = "\[Psi]";
ToUnicode[ "\\omega" ] = "\[Omega]";
ToUnicode[ "\\Gamma" ] = "\[CapitalGamma]";
ToUnicode[ "\\Delta" ] = "\[CapitalDelta]";
ToUnicode[ "\\Theta" ] = "\[CapitalTheta]";
ToUnicode[ "\\Lambda" ] = "\[CapitalLambda]";
ToUnicode[ "\\Xi" ] = "\[CapitalXi]";
ToUnicode[ "\\Pi" ] = "\[CapitalPi]";
ToUnicode[ "\\Sigma" ] = "\[CapitalSigma]";
ToUnicode[ "\\Upsilon" ] = "\[CapitalUpsilon]";
ToUnicode[ "\\Phi" ] = "\[CapitalPhi]";
ToUnicode[ "\\Psi" ] = "\[CapitalPsi]";
ToUnicode[ "\\Omega" ] = "\[CapitalOmega]";
ToUnicode[ "\\infty" ] = "\[Infinity]";
ToUnicode[ "\\pm" ] = "\[PlusMinus]";
ToUnicode[ "\\partial" ] = "\[PartialD]";
ToUnicode[ "\\leq" ] = "\[LessEqual]";
ToUnicode[ "\\geq" ] = "\[GreaterEqual]";
ToUnicode[ "\\times" ] = "\[Times]";
ToUnicode[ "\\otimes" ] = "\[CircleTimes]";
ToUnicode[ "\\oplus" ] = "\[CirclePlus]";
ToUnicode[ "\\nabla" ] = "\[Del]";
ToUnicode[ "\\neq" ] = "\[NotEqual]";
ToUnicode[ "\\equiv" ] = "\[Congruent]";
ToUnicode[ "\\approx" ] = "\[TildeTilde]";
ToUnicode[ "\\ldots" ] = "\[Ellipsis]";
ToUnicode[ "\\in" ] = "\[Element]";
ToUnicode[ "\\notin" ] = "\[NotElement]";
ToUnicode[ "\\sim" ] = "\[Tilde]";
ToUnicode[ "\\sqrt" ] = "\[Sqrt]";
ToUnicode[ "\\propto" ] = "\[Proportional]";
ToUnicode[ "\\subset" ] = "\[Subset]";
ToUnicode[ "\\supset" ] = "\[Superset]";
ToUnicode[ "\\subseteq" ] = "\[SubsetEqual]";
ToUnicode[ "\\supseteq" ] = "\[SupersetEqual]";
ToUnicode[ "\\bullet" ] = "\[FilledSmallCircle]";
ToUnicode[ "\\perp" ] = "\[RightAngle]";
ToUnicode[ "\\simeq" ] = "\[TildeEqual]";
ToUnicode[ "\\vee" ] = "\[Vee]";
ToUnicode[ "\\wedge" ] = "\[Wedge]";
ToUnicode[ "\\leftrightarrow" ] = "\[LeftRightArrow]";
ToUnicode[ "\\leftarrow" ] = "\[LeftArrow]";
ToUnicode[ "\\rightarrow" ] = ToUnicode[ "\\to" ] = "\[RightArrow]";
ToUnicode[ "\\uparrow" ] = "\[UpArrow]";
ToUnicode[ "\\downarrow" ] = "\[DownArrow]";
ToUnicode[ "\\Leftarrow" ] = "\[DoubleLeftArrow]";
ToUnicode[ "\\Rightarrow" ] = "\[DoubleRightArrow]";
ToUnicode[ "\\Uparrow" ] = "\[DoubleUpArrow]";
ToUnicode[ "\\Downarrow" ] = "\[DoubleDownArrow]";
ToUnicode[ "\\bar" ] = "-";
ToUnicode[ "\\hat" ] = "^";
ToUnicode[ "\\tilde" ] = "\[Tilde]";
ToUnicode[ "\\dot" ] = "\[CenterDot]";
ToUnicode[ "\\ddot" ] = "\[CenterDot]\[CenterDot]";
ToUnicode[ "\\vec" ] = "\[RightVector]";
ToUnicode[ "\\prime" ] = "'";
ToUnicode[ "\\#" ] = "#";
ToUnicode[ "\\&" ] = "&";
ToUnicode[ "\\$" ] = "$";
ToUnicode[ "\\%" ] = "%";
ToUnicode[ "\\_" ] = "_";
ToUnicode[ Null ] = " ";
ToUnicode[ ComposedChar[t__] ] := ToUnicode/@ {t};
ToUnicode[ c_ ] := ToString[c]

End[]

